/*
 * myS3Common.cpp
 *
 *  Created on: Jul 2, 2014
 *      Author: nick
 */

#include "myLibS3.h"
#include <cstdio>
#include <openssl/bio.h>
#include <openssl/evp.h>
#include <openssl/hmac.h>
#include <cstring>
#include <sstream>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <algorithm>
#include <cctype>
#include <errno.h>
#include "myS3Common.h"

using namespace std;

extern int errno;

char secretKey[32];
char privateSecretKey[128];
bool MyS3Context::bKeyInitialized = false;

bool MyS3Context::initialize(const char* pszBucket, S3BucketContext& ctx)
{
	if (!bKeyInitialized)
	{
		fstream of("/home/nick/.amazon/s3key", ios_base::in);
		if (of.good())
		{
			of.getline(secretKey, sizeof(secretKey));
			of.getline(privateSecretKey, sizeof(privateSecretKey));
			bKeyInitialized = true;
		}
		else
		{
			return false;
		}
	}

	ctx.accessKeyId = secretKey;
	ctx.secretAccessKey = privateSecretKey;
	ctx.bucketName = pszBucket;
	//ctx.hostName = "s3.amazon.com";
	ctx.hostName = NULL;
	ctx.protocol = S3ProtocolHTTPS;
	ctx.uriStyle = S3UriStylePath;
	//ctx.uriStyle = S3UriStyleVirtualHost;
	bInitialized = S3_initialize("ubuntu 10.04", 0, NULL) == S3StatusOK;
	return bInitialized;
}

MyS3Context::~MyS3Context()
{
	if (bInitialized)
	{
		S3_deinitialize();
	}
}

void printS3Obj(const S3Obj& s3Obj)
{
	cout << "key:" << s3Obj.key << endl
		<< "size:" << s3Obj.size << endl
		<< "time:" << s3Obj.time << endl
		<< "tag:" << s3Obj.tag << endl;
}

void printS3ObjVect(const S3ObjVect& s3ObjVect)
{
	for (size_t i = 0; i < s3ObjVect.size(); i ++)
	{
		printS3Obj(s3ObjVect[i]);
	}
}

void printS3ObjSet(const S3ObjSet& s3ObjSet)
{
	for (S3ObjSet::const_iterator it = s3ObjSet.begin(); it != s3ObjSet.end(); it ++)
	{
		printS3Obj(*it);
	}
}

S3Status MyS3ResponsePropertiesCallback(const S3ResponseProperties *properties,
		void *callbackData)
{
	return S3StatusOK;
}

void MyS3ResponseCompleteCallback(S3Status status,
				  const S3ErrorDetails *errorDetails,
				  void *callbackData)
{
	if (status != S3StatusOK)
	{
		if (errorDetails)
		{
			cout << errorDetails->extraDetails << endl
					<< errorDetails->furtherDetails << endl
					<< errorDetails->message << endl
					<< errorDetails->resource << endl
					<< errorDetails->extraDetailsCount << endl;
		}
	}
}

bool getFileSize(const string& strFileName, size_t& size)
{
	struct stat buf = {0};
	if (stat(strFileName.c_str(), &buf) == 0)
	{
		if ((S_IFMT & buf.st_mode) == S_IFREG)
		{
			size = buf.st_size;
			return true;
		}
	}
	return false;
}

string int2string(int number)
{
    stringstream stream;
    stream << number;
    return stream.str();
}
char myToUpper(char ch)
{
	return toupper(ch);
}
// I make a helper function to compare string
int CompareCaseInsensitive(string strFirst, string strSecond)
{
	return StringManager::icompare(strFirst, strSecond);
}



unsigned char ch2hex(unsigned char ch)
{
	if (ch > 9)
	{
		return ch - 10 + 'a';
	}
	else
	{
		return ch + '0';
	}
}

bool readKeyFile(const string& keyPath)
{
	FILE* fd = fopen(keyPath.c_str(), "r");
	if (fd)
	{
		fgets(secretKey, 32, fd);
		secretKey[strlen(secretKey) - 1]= '\0';
		fgets(privateSecretKey, 64, fd);
		privateSecretKey[strlen(privateSecretKey) - 1] = '\0';
		fclose(fd);
		return true;
	}
	return false;
}

string convertToHex(const string& strIn)
{
	string str;
	const unsigned char MASKHI = 0xF0;
	const unsigned char MASKLO = 0x0F;
	for (size_t i = 0; i < strIn.size(); i ++)
	{
		unsigned char hi = (MASKHI & strIn[i]) >> 4;
		unsigned char lo = MASKLO & strIn[i];
		str.push_back(ch2hex(hi));
		str.push_back(ch2hex(lo));
	}
	return str;
}

MyMD5::MyMD5(bool bHexOutput)
{
	bHex = bHexOutput;
	MD5_Init(&ctx);
}

void MyMD5::update(const string& strIn)
{
	MD5_Update(&ctx, strIn.c_str(), strIn.size());
}

string MyMD5::finalize()
{
	unsigned char raw_digest[16];
	MD5_Final(raw_digest, &ctx);

	if (bHex)
	{
		return convertToHex(string((char*)raw_digest, 16));
	}
	return string((char*)raw_digest, 16);
}

MyBase64Encoder::MyBase64Encoder()
{
	b64 = BIO_new(BIO_f_base64());
	bio = BIO_new( BIO_s_mem() );
	BIO_set_flags(b64, BIO_FLAGS_BASE64_NO_NL);
	bio = BIO_push(b64, bio);
	strEncoded.clear();
}

void MyBase64Encoder::update(const string& strIn)
{
	BIO_write(bio, strIn.c_str(), strIn.size());
	BIO_flush(bio);
	char* ptrOut = NULL;
	size_t rLen = BIO_get_mem_data(bio, &ptrOut);
	if (ptrOut)
	{
		strEncoded.append(ptrOut, rLen);
	}

}
string MyBase64Encoder::finalize()
{
	BIO_free_all(bio);
	return strEncoded;
}


MyBase64Decoder::MyBase64Decoder()
{
	b64 = BIO_new(BIO_f_base64());
	strDecoded.clear();
}

void MyBase64Decoder::update(const string& strIn)
{
	size_t nTotalToDecode = strIn.size(), nTotalDecoded = 0, nDecoded = 0, nToDecode = 0;
	while (nTotalToDecode > 0)
	{
		if (nTotalToDecode > MyBase64Decoder_Internal_Buffer_Size)
		{
			nToDecode = MyBase64Decoder_Internal_Buffer_Size;
		}
		else
		{
			nToDecode = nTotalToDecode;
		}
		memset(buffer, 0, nToDecode);
		bmem = BIO_new_mem_buf((void*)(strIn.c_str() + nTotalDecoded), nToDecode);
		bmem = BIO_push(b64, bmem);
		nDecoded = BIO_read(bmem, buffer, nToDecode);
		strDecoded.append((char*)buffer, nDecoded);

		nTotalToDecode -= nToDecode;
		nTotalDecoded += nToDecode;
	}
}

string MyBase64Decoder::finalize()
{
	BIO_free_all(bmem);
	return strDecoded;
}

MySH1::MySH1(const string& strKeyIn, bool bHexOutput)
{
	bHex = bHexOutput;
	strKey = strKeyIn;
	HMAC_CTX_init(&theHctx);
	HMAC_Init(&theHctx, strKey.c_str(), strKey.size(), EVP_sha1());
	strSigned.clear();
}

void MySH1::update(const string& strIn)
{
	size_t nTotalToSign = strIn.size(), nTotalSigned = 0, nToSign = 0;
	unsigned int nSigned = 0;
	while (nTotalToSign > 0)
	{
		if (nTotalToSign > MySH1_Internal_Buffer_Size)
		{
			nToSign = MySH1_Internal_Buffer_Size;
		}
		else
		{
			nToSign = nTotalToSign;
		}
		HMAC(EVP_sha1(), strKey.c_str(), strKey.size(), (unsigned char*)(strIn.c_str() + nTotalSigned), nToSign, buffer, &nSigned);
		strSigned.append((char*)buffer, nSigned);
		nTotalSigned += nToSign;
		nTotalToSign -= nToSign;
	}
}

string MySH1::finalize()
{
	HMAC_CTX_cleanup(&theHctx);
	if (bHex)
	{
		return convertToHex(strSigned);
	}
	return strSigned;
}



bool ci_char_traits::eq(char c1, char c2)
{
    return toupper(c1) == toupper(c2);
}
bool ci_char_traits::ne(char c1, char c2)
{
    return toupper(c1) != toupper(c2);
}
bool ci_char_traits::lt(char c1, char c2)
{
    return toupper(c1) <  toupper(c2);
}


int ci_char_traits::compare(const char* s1, const char* s2, size_t n)
{
    while ( n-- != 0 )
    {
        if ( toupper(*s1) < toupper(*s2) ) return -1;
        if ( toupper(*s1) > toupper(*s2) ) return 1;
        ++s1;
        ++s2;
    }
    return 0;
}

const char* ci_char_traits::find(const char* s, int n, char a)
{
    while ( n-- > 0 && toupper(*s) != toupper(a) )
    {
        ++s;
    }
    return s;
}

string StringManager::int2string(int number)
{
    stringstream stream;
    stream << number;
    return stream.str();
}

int StringManager::icompare(const std::string& left, const std::string& right)
{
    if (left.size() != right.size())
    {
        return left.size() - right.size();
    }
    else
    {
        for (size_t i = 0; i < left.size(); i ++)
        {
            char l = toupper(left[i]), r = toupper(right[i]);

            if ( l != r )
            {
                return l - r;
            }
        }
        return 0;
    }
}

int StringManager::incompare(const std::string& left, const std::string& right, size_t len)
{
    size_t length = len;
    if (length > left.size())
    {
        length = left.size();
    }
    if (length > right.size())
    {
        length = right.size();
    }

    for (size_t i = 0; i < length; i ++)
    {
        char l = toupper(left[i]), r = toupper(right[i]);
        if ( l != r )
        {
            return l - r;
        }
    }
    return 0;
}


bool string2int(const std::string& str, int& number)
{
    return sscanf(str.c_str(), "%d", &number) == 1;
}

bool calculateMD5Base64(const string& strPath, string& strResult)
{
#define MYS3_FILE_BUFFER_SIZE 1024
	FILE* stream = fopen(strPath.c_str(), "r");
	if (stream)
	{
		char buffer[MYS3_FILE_BUFFER_SIZE];
		MyMD5 myMD5(false);
		int readSize;

		while ((readSize = fread(buffer, 1, MYS3_FILE_BUFFER_SIZE, stream)) > 0)
		{
			string str(buffer, readSize);
			myMD5.update(str);

		}
		fclose(stream);

		MyBase64Encoder enc;
		enc.update(myMD5.finalize());
		strResult =  enc.finalize();
		return true;
	}
	return false;
}

bool calculateMD5Hex(const string& strPath, string& strResult)
{
#define MYS3_FILE_BUFFER_SIZE 1024
	FILE* stream = fopen(strPath.c_str(), "r");
	if (stream)
	{
		char buffer[MYS3_FILE_BUFFER_SIZE];
		MyMD5 myMD5(true);
		int readSize;

		while ((readSize = fread(buffer, 1, MYS3_FILE_BUFFER_SIZE, stream)) > 0)
		{
			string str(buffer, readSize);
			myMD5.update(str);
		}
		fclose(stream);
		strResult =  myMD5.finalize();
		return true;
	}
	return false;
}

bool populateS3Obj(const string& strPath, const string& strKey, S3Obj& s3Obj)
{
	struct stat buf = {0};
	if (stat(strPath.c_str(), &buf) == 0)
	{
		if ((S_IFMT & buf.st_mode) == S_IFREG)
		{
			s3Obj.key = strKey;
			s3Obj.size = buf.st_size;
			s3Obj.time = buf.st_mtim.tv_sec;
			string strMD5;
			if (calculateMD5Hex(strPath, strMD5))
			{
				s3Obj.tag = strMD5;
				return true;
			}
		}
	}
	return false;
}


string trimQuote(const string& str)
{
	if (str.size() > 0 && (str[0] == '"' || str[0] == '\'') && (str[str.size()-1] == '"' || str[str.size()-1]=='\''))
	{
		return str.substr(1, str.size()-2);
	}
	return str;
}

bool ensureDirectory(const string& strPath)
{
	if (strPath.size() == 0)
	{
		return false;
	}

	struct stat buf = {0};
	if (lstat(strPath.c_str(), &buf) == 0)
	{
		if (S_ISDIR(buf.st_mode))
		{
			return true;
		}
		return false;
	}
	else
	{
		if (errno == ENOENT)
		{
			size_t pos = strPath.find_last_of('/');
			if (pos != string::npos)
			{
				if (ensureDirectory(strPath.substr(0, pos)))
				{
					if (mkdir(strPath.c_str(), S_IRWXU|S_IRWXO|S_IRWXG) == 0)
					{
						return true;
					}
					if (errno == EEXIST)
					{
						return true;
					}
					else
					{
						perror("mkdir error");
					}
				}
			}
		}
	}
	return false;
}

bool ensureParentDirectory(const string& strPath)
{
	string str = strPath;
	if (strPath.size() > 1 && strPath[0] != '/')
	{
		char buffer[2048];
		if (!getcwd(buffer, 2048))
		{
			return false;
		}
		str =  buffer;
		str += "/";
		str += strPath;
	}
	size_t pos = str.find_last_of('/');
	if (pos != string::npos)
	{
		return ensureDirectory(str.substr(0, pos));
	}
	return false;
}
