/*
 * myS3Common.h
 *
 *  Created on: Jul 3, 2014
 *      Author: nick
 */

#ifndef __MYS3COMMON_H__
#define __MYS3COMMON_H__

#include <openssl/md5.h>
#include <openssl/hmac.h>
#include <string>
#include <vector>
#include <cstdio>
#include "myLibS3.h"


#define MySH1_Internal_Buffer_Size 256
#define MyBase64Decoder_Internal_Buffer_Size 256


std::string int2string(int number);
bool string2int(const std::string& str, int& number);
bool getFileSize(const std::string& strFileName, size_t& size);
bool readKeyFile(const std::string& keyPath);
std::string getFileContentType(const std::string& fileName);
bool calculateMD5Base64(const std::string& strPath, std::string& strResult);
bool calculateMD5Hex(const std::string& strPath, std::string& strResult);
bool populateS3Obj(const std::string& strPath, const std::string& strKey, S3Obj& s3Obj);
void printS3ObjVect(const S3ObjVect& s3ObjVect);
void printS3ObjSet(const S3ObjSet& s3ObjSet);
std::string trimQuote(const std::string& str);

bool ensureParentDirectory(const std::string& strPath);
extern std::string getFileContentType(const std::string& fileName);

class MyS3Context
{
public:
	bool initialize(const char* pszBucket, S3BucketContext& ctx);
	~MyS3Context();
private:
	static bool bKeyInitialized;
	bool bInitialized;
	S3BucketContext ctx;
};

class MyBase64Encoder
{
private:
	std::string strEncoded;
	BIO *bio;
	BIO *b64;
public:
	MyBase64Encoder();
	void update(const std::string& strIn);
	std::string finalize();
};


class MyBase64Decoder
{
private:
	std::string strDecoded;
	BIO* b64;
	BIO* bmem;
	unsigned char buffer[MyBase64Decoder_Internal_Buffer_Size];
public:
	MyBase64Decoder();
	void update(const std::string& strIn);
	std::string finalize();
};

class MySH1
{
private:
	HMAC_CTX theHctx;
	std::string strKey;
	bool bHex;
	std::string strSigned;
	unsigned char buffer[MySH1_Internal_Buffer_Size];
public:
	MySH1(const std::string& strKeyIn, bool bHexOutput = false);
	void update(const std::string& strIn);
	std::string finalize();
};

class MyMD5
{
private:
	MD5_CTX ctx;
	bool bHex;
public:
	MyMD5(bool bHexOutput = true);
	void update(const std::string& strIn);
	std::string finalize();
};

typedef std::vector<std::string> StringVector;
class StringManager
{
public:
	static StringVector split(const std::string& src, const char* delimiter=" \t\n");
	static std::string trim(const std::string& src, const char* delimiter=" \t\n");
	static std::string int2string(int num);
	static bool string2int(const std::string& str, int& num);
	static int icompare(const std::string& left, const std::string& right);
	static int incompare(const std::string& left, const std::string& right, size_t len);
};

struct ci_char_traits : public std::char_traits<char>
{
	static bool eq(char c1, char c2);

	static bool ne(char c1, char c2);

	static bool lt(char c1, char c2);

	static int compare(const char* s1, const char* s2, size_t n);

	static const char* find(const char* s, int n, char a);
};

typedef std::basic_string<char, ci_char_traits> ci_string;

#endif /* MYS3COMMON_H_ */
